﻿#region License Information

/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */

#endregion

using System;
using System.CodeDom;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking;
using HeuristicLab.Problems.BinPacking2D;

namespace HeuristicLab.Problems.BinPacking2d {
  [StorableClass]
  [Creatable(Category = CreatableAttribute.Categories.CombinatorialProblems, Priority = 300)]
  public class PackingSequenceProblem : PackingSequenceProblem<Permutation, PermutationEncoding> {

    // persistence
    [StorableConstructor]
    protected PackingSequenceProblem(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }

    public override bool Maximization {
      get { return true; }
    }


    // cloning
    protected PackingSequenceProblem(PackingSequenceProblem original, Cloner cloner)
      : base(original, cloner) {
    }

    protected PackingSequenceProblem() : base() { }


    public override IDeepCloneable Clone(Cloner cloner) {
      return new PackingSequenceProblem(this, cloner);
    }


    public override double Evaluate(Individual individual, IRandom random) {
      var permutation = individual.Permutation();
      var decoder = new ExtremePointPackingSequenceDecoder2D();
      var packingPlan = decoder.CreatePackingPlanFromEncoding(permutation, Bin, Items);
      var ratio = PackingRatioEvaluator.CalculatePackingRatio(packingPlan);
      return ratio.Value;
    }

    public override void Analyze(Individual[] individuals, double[] qualities, ResultCollection results, IRandom random) {
      base.Analyze(individuals, qualities, results, random);
      Analyze(individuals.Select(i => i.Permutation()).ToArray(), qualities, results, random);
    }
  }
}
